__author__ = 'Martin'

import sys
from my_OOP_stack import stack as Stack
from array_types import array_, copy_left, copy_right
from collections.abc import Iterable

DEFAULT_CAPACITY = 2    # use a low value for testing

class List(Stack):
    def __init__(self, data=None):
        if isinstance(data, Iterable) and len(data) > 0:
            data_len = len(data)
            self.capacity = data_len * 3 // 2
            super().__init__(self.capacity)
            copy_left(data, 0, self.array, 0, data_len)
            self.SP = data_len
        else:
            super().__init__(DEFAULT_CAPACITY)
            self.capacity = DEFAULT_CAPACITY

        self.max_size = sys.maxsize         # set inherited max_size to a very large number

    def _check_capacity(self):
        if self.SP < self.capacity - 1:     # leave a spare element for a guard
            return
        self.capacity = self.capacity * 3 // 2
        new_array = array_(self.capacity)
        copy_left(self.array, 0, new_array, 0, self.SP)
        self.array = new_array

    def push(self, item):
        self._check_capacity()
        super().push(item)

    def insert_at(self, index, item):
        self._check_capacity()
        to_move = self.SP - index
        array = self.array
        copy_right(array, index, array, index+1, to_move)
        array[index] = item
        self.SP += 1

    def delete_at(self, index):
        array = self.array
        deleted = array[index]
        to_move = self.SP - index - 1   # one less than insert - not moving the deletion
        copy_left(array, index+1, array, index, to_move)
        self.SP -= 1
        return deleted

    def add_guard(self, item):
        self._check_capacity()
        self.array[self.SP] = item


if __name__ == '__main__':

    t = List()
    for p in range(0, 4):  # i.e. 2..26
        t.push(p)
    print(t.array)
    t.delete_at(0)
    t.push(4)
    print(t.array)

    t = List('abcd')
    print(t.array)

    # t = List()
    # t.push(0)
    # print(t.array)
    # t.push(1)
    # print(t.array)
    # for p in range(2, 27): # i.e. 2..26
    #     t.push(p)
    # print(t.array)
    # t.add_guard(27)
    # print(t.array)
    # t.insert_at(0, 99)
    # print(t.array)
    # t.insert_at(27, 27)
    # print(t.array)
    # print(t.delete_at(0))
    # print(t.array)
    # t.push(33)
    # print(t.array)
    # print(t.delete_at(27))
    # print(t.array)
    # print(t.SP)
